function A = bnet_mdds(nu)
%bnet_mdds --> Computation of the Bezier net coefficients 
%                of an arbitrary bivariate three-direction/
%                trivariate four-direction Box-spline.
%
%<Synopsis>
%  A = bnet_mdds(nu)
%
%<Description>
%  Implements the mixed directional difference-summation 
%  algorithm (MDDS) for generating the Bezier net either of a 
%  bivariate three-direction or a trivariate four-direction 
%  Box-spline, associated with the direction matrix D.  
%
%  In the bivariate case D is 
%
%          nu(1)          nu(2)           nu(3)      
%      ------^------  ------^------  -------^-------  
%   D=[e_1, ..., e_1, e_2, ..., e_2, e_12, ..., e_12].
%
%  Here, the input parameter nu is a 3-component vector 
%  with the multiplicities of the directions
%  e_1=[1 0]', e_2=[0 1]', e_12=[1 1]':
%
%  nu(1) : multiplicity of direction e_1
%  nu(2) : multiplicity of direction e_2
%  nu(3) : multiplicity of direction e_12.
%
%  In the trivariate case D is instead
%
%          nu(1)          nu(2)          nu(3)            nu(4)
%      ------^------  ------^------  ------^------  --------^--------
%   D=[e_1, ..., e_1, e_2, ..., e_2, e_3, ..., e_3, e_123, ..., e_123].
%
%  Here, the input parameter nu is a 4-component vector 
%  with the multiplicities of the directions
%  e_1=[1 0 0]', e_2=[0 1 0]', e_3=[0 0 1]', e_123=[1 1 1]': 
%
%  nu(1) : multiplicity of direction e_1
%  nu(2) : multiplicity of direction e_2
%  nu(3) : multiplicity of direction e_3
%  nu(4) : multiplicity of direction e_123.
%
%<Input Parameters>          
%       |--> 3-component vector (if a bivariate B-net is required)
%   nu  | 
%       |--> 4-component vector (if a trivariate B-net is required)
%
%<Output Parameters>
%       |--> bi-dimensional array containing the B-net coefficients 
%       |    of the required bivariate three-direction Box-spline;
%   A   |    if deg=sum(nu)-2 denotes the Box-spline degree, then
%       |    size(A)=[deg(nu(2)+nu(3))+1]x[deg(nu(1)+nu(3))+1].
%       |
%       |--> tri-dimensional array containing the B-net coefficients 
%            of the required trivariate four-direction Box-spline;
%            if deg=sum(nu)-3 denotes the Box-spline degree, then
%            size(A)=[deg(nu(3)+nu(4))+1]x[deg(nu(1)+nu(4))+1]x[deg(nu(2)+nu(4))+1].

%<Algorithm>
%  Starting from the Bezier net of the degree-1 Box-spline, the Bezier net of  
%  the required bivariate/trivariate Box-spline is generated by successively 
%  computing differences-summations of the Bezier coefficients along the 
%  three/four directions set. These procedures are implemented respectively in 
%  {step1_mdds2d, step2_mdds2d, step3_mdds2d},
%  {step1_mdds3d, step2_mdds3d, step3_mdds3d, step4_mdds3d}.
%
%<Reference>
%  [1] G. Casciola, E. Franchini, L. Romani, "The mixed directional 
%      difference-summation algorithm for generating the Bezier net of a 
%      trivariate four-direction Box-spline"
%      Numerical Algorithms 43 (2006), pp. 75-98.
%
%<Authors>
%  G. Casciola, Dept. of Mathematics - University of Bologna, Italy
%  E. Franchini, Dept. of Pure and Applied Mathematics - University of Padova, Italy
%  L. Romani, Dept. of Mathematics - University of Bologna, Italy
%
%  Last revision: March 22, 2006

% Check the required input arguments
if nargin < 1
    error('Not enough input arguments.')
elseif nargin > 1
    error('Too many input arguments.')    
end

nu=nu(:);
ln=length(nu);

if ln==3
   flag=1;
elseif ln==4
   flag=2;
else
   error('The multiplicities vector nu is either a 3-component or a 4-component vector.');
end

switch flag
       case 1
            % Bezier net of the degree-1 bivariate three-direction Box-spline    
            B=[0 0 0;0 1 0;0 0 0];
    
            if all(nu==ones(ln,1))
               A=B;
            end
    
            % Step_1 : differences-summations along direction e_1=[1 0]'
            for i=2:nu(1)
                A=step1_mdds2d(i,B);
                B=A;
            end
            % Step_2 : differences-summations along direction e_2=[0 1]'
            for i=2:nu(2)
                A=step2_mdds2d(nu(1),i,B);
                B=A;
            end
            % Step_3 : differences-summations along direction e_12=[1 1]'
            for i=2:nu(3)
                A=step3_mdds2d(nu(1),nu(2),i,B);
                B=A;
            end
       case 2
            % Bezier net of the degree-1 trivariate four-direction Box-spline
            B(:,:,1)=zeros(3,3);
            B(:,:,2)=[0 0 0;0 1 0;0 0 0];
            B(:,:,3)=zeros(3,3);
    
            if all(nu==ones(ln,1))
               A=B;
            end
    
            % Step_1 : differences-summations along direction e_1=[1 0 0]'
            for i=2:nu(1)
                A=step1_mdds3d(i,B);
                B=A; 
            end
            % Step_2 : differences-summations along direction e_3=[0 0 1]'
            for i=2:nu(3)
                A=step2_mdds3d(nu(1),i,B);
                B=A;
            end
            % Step_3 : differences-summations along direction e_2=[0 1 0]'
            for i=2:nu(2)
                A=step3_mdds3d(nu(1),i,nu(3),B);
                B=A;
            end
            % Step_4 : differences-summations along direction e_123=[1 1 1]'
            for i=2:nu(4)
                A=step4_mdds3d(nu(1),nu(2),nu(3),i,B);
                B=A;
            end     
end
