function demo_mdds3d
%demo_mdds3d --> Demonstration of B-net computation, exact evaluation and
%                visualization of trivariate four-direction Box-splines.
%
%<See Also>
%  demo_mdds2d --> Demonstration of B-net computation, exact evaluation and
%                 visualization of bivariate three-direction Box-splines.

%<Reference>
%  [1] G. Casciola, E. Franchini, L. Romani, "The mixed directional 
%      difference-summation algorithm for generating the Bezier net of a 
%      trivariate four-direction Box-spline"
%      Numerical Algorithms 43 (2006), pp. 75-98.
%
%<Authors>
%  G. Casciola, Dept. of Mathematics - University of Bologna, Italy
%  E. Franchini, Dept. of Pure and Applied Mathematics - University of Padova, Italy
%  L. Romani, Dept. of Mathematics - University of Bologna, Italy
%
%  Last revision: March 22, 2006

clc

disp('|---------------------------------------------------------------------|');
disp('| demo_mdds3d                                                         |');
disp('| This function provides three examples to demonstrate the use of the |');
disp('| functions bnet_mdds(), pointeval_mdds3d(), visual_mdds3d() which    |');
disp('| allow respectively the Bezier net computation, the stable and       |');
disp('| efficient evaluation in an arbitrary set of 3D points, and the      |');
disp('| visualization of the s-set (s in Imm(M_D)) extraction and the       |'); 
disp('| contour lines of a given trivariate four-direction Box-spline       |'); 
disp('| volume M_D.                                                         |');
disp('|---------------------------------------------------------------------|');

disp(' ');
disp('The first example uses the cubic Box-spline M_2112');
disp(' ');
disp('The multiple occurrence of identical directions e_1=[1 0 0]'','); 
disp('e_2=[0 1 0]'', e_3=[0 0 1]'', e_123=[1 1 1]'' is logged in a vector nu'); 
disp('holding the number of times each direction is repeated:');
disp(' ');
nu=[2 1 1 2];
fprintf('nu= [%d %d %d %d]\n\n',nu);
fprintf('Press any key to continue.\n\n');
pause  

disp('1. Computation of the Bezier net coefficients of the cubic'); 
disp('   Box-spline M_2112');
A1=bnet_mdds(nu);
m=sum(nu)-3;
disp(' ');
disp('The matrix you see right now is exactly the tri-dimensional array'); 
disp('containing the Bezier net coefficients of the cubic Box-spline M_2112');
fprintf('(save for the factor 1/m!, where m=%d is degree(M_2112)\n\n',m);
% size(A1)=[m(nu(3)+nu(4))+1]x[m(nu(1)+nu(4))+1]x[m(nu(2)+nu(4))+1]
for i=1:m*(nu(2)+nu(4))+1
    fprintf('B-net matrix (:,:,%d)\n',i);
    disp(A1(:,:,i));
    fprintf('\nPress any key to continue.\n\n');
    pause   
end
disp(' '); 

disp('2. Evaluation of the cubic Box-spline M_2112');
disp(' ');
disp('The evaluation algorithm can be applied to an arbitrary set of sample');
disp('3D points inside the Box-spline support. Here we evaluate the cubic'); 
disp('Box-spline M_2112 in a given arbitrary 3D point P:');
disp(' ');
P=[1.2 1.0 1.1];
fprintf('P=[%2.1f %2.1f %2.1f]\n',P);
point_value=pointeval_mdds3d(nu,P);
fprintf('point_value=%6.4f\n\n',point_value);
fprintf('Press any key to continue.\n\n');
pause  

disp('3. Visualization of the cubic Box-spline M_2112');
disp(' ');
disp('The Box-spline volume M_2112 can be visualized either through the'); 
disp('s-set (s in Imm(M_D)) extraction of the Box-spline volume or through'); 
disp('the contour lines of the regions obtained by intersecting the'); 
disp('Box-spline volume with three families of planes respectively parallel'); 
disp('to yz, xz, xy.');
disp(' ');
disp('The isosurfaces you see right now are respectively the graphs of'); 
disp('the 0-set extraction (see Fig.11 left in [1])');
visual_mdds3d(nu,1,8,0);
fprintf('\nPress any key to continue.\n\n');
pause   

clf

visual_mdds3d(nu,1,8,0.01);
disp('and the 0.01-set extraction (see Fig.11 right in [1]) of the'); 
disp('Box-spline volume M_2112'); 
fprintf('\nPress any key to continue.\n\n');
pause  

disp('The following figures contain the contour lines of the');
disp('regions obtained by intersecting the Box-spline volume');
disp('with three families of planes respectively parallel to yz, xz, xy');
visual_mdds3d(nu,2,8);
fprintf('Press any key to see next example.\n\n');
pause  

clc

disp('--------------------------------------------------------------------');
disp(' ');
disp('The second example uses the quintic Box-spline M_2222');
disp('associated with the multiplicities vector nu:');
disp(' ');
nu=[2 2 2 2];
fprintf('nu= [%d %d %d %d]\n\n',nu);
fprintf('Press any key to continue.\n\n');
pause  

disp('1. Computation of the Bezier net coefficients of the quintic'); 
disp('   Box-spline M_2222');
A1=bnet_mdds(nu);
m=sum(nu)-3;
disp(' ');
disp('The matrix you see right now is exactly the tri-dimensional array'); 
disp('containing the Bezier net coefficients of the quintic Box-spline M_2222');
fprintf('(save for the factor 1/m!, where m=%d is degree(M_2222)\n\n',m);
% size(A1)=[m(nu(3)+nu(4))+1]x[m(nu(1)+nu(4))+1]x[m(nu(2)+nu(4))+1]
for i=1:m*(nu(2)+nu(4))+1
    fprintf('B-net matrix (:,:,%d)\n',i);
    disp(A1(:,:,i));
    fprintf('\nPress any key to continue.\n\n');
    pause  
end

disp('2. Evaluation of the quintic Box-spline M_2222');
disp('   in a given arbitrary 3D point P:');
disp(' ');
P=[1.0 1.0 1.0];
fprintf('P=[%2.1f %2.1f %2.1f]\n',P);
point_value=pointeval_mdds3d(nu,P);
fprintf('point_value=%6.4f\n\n',point_value);
fprintf('Press any key to continue.\n\n');
pause   

disp('3. Visualization of the quintic Box-spline M_2222');
disp(' ');
disp('The Box-spline volume M_2222 can be visualized either through the'); 
disp('s-set (s in Imm(M_D)) extraction of the Box-spline volume or through'); 
disp('the contour lines of the regions obtained by intersecting the'); 
disp('Box-spline volume with three families of planes respectively parallel'); 
disp('to yz, xz, xy.');
disp(' ');
disp('The isosurfaces you see right now are respectively the graphs of'); 
disp('the 0-set extraction (see Fig.12 left in [1])');
visual_mdds3d(nu,1,8,0);
fprintf('\nPress any key to continue.\n\n');
pause  

clf

visual_mdds3d(nu,1,8,0.005);
disp('and the 0.005-set extraction (see Fig.12 right in [1]) of the'); 
disp('Box-spline volume M_2222'); 
fprintf('\nPress any key to continue.\n\n');
pause   

disp('The following figures contain the contour lines of the');
disp('regions obtained by intersecting the Box-spline volume');
disp('with three families of planes respectively parallel to yz, xz, xy');
visual_mdds3d(nu,2,8);
fprintf('\nPress any key to see next example.\n\n');
pause  

clc

disp('-------------------------------------------------------------------------');
disp(' ');
disp('The last example uses the sextic Box-spline M_3222'); 
disp('associated with the following multiplicities vector nu:');
disp(' ');
nu=[3 2 2 2];
fprintf('nu= [%d %d %d %d]\n\n',nu);
fprintf('Press any key to continue.\n\n');
pause  

disp('1. Computation of the Bezier net coefficients of the sextic'); 
disp('   Box-spline M_3222');
A2=bnet_mdds(nu);
m=sum(nu)-3;
disp(' ');
disp('The matrix you see right now is exactly the tri-dimensional array'); 
disp('containing the Bezier net coefficients of the sextic Box-spline M_3222');
fprintf('(save for the factor 1/m!, where m=%d is degree(M_3222)\n\n',m);
% size(A2)=[m(nu(3)+nu(4))+1]x[m(nu(1)+nu(4))+1]x[m(nu(2)+nu(4))+1]
for i=1:m*(nu(2)+nu(4))+1
    fprintf('B-net matrix (:,:,%d)\n',i);
    disp(A2(:,:,i));
    fprintf('\nPress any key to continue.\n\n');
pause  
end

disp('2. Evaluation of the sextic Box-spline M_3222 in a given arbitrary'); 
disp('   3D point P');
P=[1.0 1.25 0.75];
fprintf('P=[%2.2f %2.2f %2.2f]\n',P);
point_value=pointeval_mdds3d(nu,P);
fprintf('point_value=%6.4f\n\n',point_value);
fprintf('Press any key to continue.\n\n');
pause  

disp('3. Visualization of the sextic Box-spline M_3222');
disp(' ');
disp('Visualization of the 0-set extraction of the Box-spline volume M_3222');
visual_mdds3d(nu,1,8,0);
fprintf('\nPress any key to continue.\n\n');
pause    

clf

visual_mdds3d(nu,1,8,0.001);
disp('Visualization of the 0.001-set extraction of the Box-spline volume'); 
disp('M_3222');
fprintf('\nPress any key to continue.\n\n');
pause  

disp('The following figures contain the contour lines of the');
disp('regions obtained by intersecting the Box-spline volume');
disp('with three families of planes respectively parallel to yz, xz, xy');
visual_mdds3d(nu,2,8);
fprintf('\nPress any key to close all.\n\n');
pause  
close all
clc
