function A = step2_mdds2d(k,L,B)
%step2_mdds2d --> Computation of differences-summations 
%                 along direction e_2=[0 1]'.
%
%<Synopsis>
%  A = step2_mdds2d(k,L,B)
%
%<Description>
%  To compute the Bezier net coefficients of the bivariate 
%  three-direction Box-spline with multiplicities vector nu=[k L 1], 
%  we generate the mixed directions that will be used for defining the 
%  integration paths on the coefficients in B. Here, B is a 
%  bi-dimensional array with the Bezier coefficients of the bivariate 
%  three-direction Box-spline with multiplicities vector [k L-1 1]. 
%
%<Input Parameters>
%   k --> nu(1) (multiplicity of direction e_1=[1 0]')
%   L --> nu(2) (multiplicity of direction e_2=[0 1]')
%   B --> bi-dimensional array with the Bezier coefficients of
%         the bivariate three-direction Box-spline with multiplicities
%         vector [k L-1 1]
%
%<Output Parameters>  
%   A --> bi-dimensional array with the Bezier coefficients of
%         the bivariate three-direction Box-spline with multiplicities
%         vector [k L 1]

%<Algorithm> 
%  First apply the shifting and subtracting procedure on the 
%  coefficients in B. Then compute the mixed directions that will be 
%  used for defining the integration paths on the new coefficients in B.
%
%<Authors>
%  G. Casciola, Dept. of Mathematics - University of Bologna, Italy
%  E. Franchini, Dept. of Pure and Applied Mathematics - University of Padova, Italy
%  L. Romani, Dept. of Mathematics - University of Bologna, Italy
%
%  Last revision: March 22, 2006

% Box-spline degree
m=k+L-1;

% Shifting and subtracting procedure on the Bezier coefficients in B
[r,col]=size(B);
dB=m-1;
B=[B;zeros(dB,col)]-[zeros(dB,col);B];

A=zeros(m*(L+1)+1,m*(k+1)+1);

% Computation of the mixed directions that will be used for defining 
% the integration paths on the new coefficients in B

% Mixed directions in R
%---------------------------------
for i=1:k
    A(:,i*m+1)=integ_mdds(dupl_mdds(B(:,i*dB+1),m,2,L));
end

% Direction pattern for the generation of mixed directions in R^2
t=[zeros(1,m-2) 1 ones(1,m-2)];

% Mixed directions in R^2
%---------------------------------
for g=0:k
    x=g*dB+2;
    for r=0:dB-1
        rr=r+1;
        d=dB+r;
        h=1;
        i=1;
        xx=x+r;
        for p=0:L
            f(h,1)=B(i,xx);
            h=h+1;    
            for v=rr:d
                i=i+1;
                f(h,1)=B(i,xx-t(v));
                h=h+1;
            end
        end
        A(:,2+g*m+r)=integ_mdds(f);      
    end  
end        