function visual_mdds3d(nu,flag,m,s)
%visual_mdds3d --> Visualization of any trivariate four-direction
%                  Box-spline M_D either through the s-set 
%                  (s in Imm(M_D)) extraction of the 
%                  Box-spline volume or through the contour
%                  lines of the regions obtained by intersecting
%                  the Box-spline volume with three families of
%                  planes respectively parallel to yz, xz, xy.
%
%<Synopsis>
%  visual_mdds3d(nu)
%  visual_mdds3d(nu,flag)
%  visual_mdds3d(nu,flag,m)
%  visual_mdds3d(nu,flag,m,s)
%
%<Description>
%  Exploits the mixed directional difference-summation 
%  algorithm (MDDS) generating the Bezier net of a 
%  trivariate four-direction Box-spline M_D associated with 
%  the direction matrix D
%
%          nu(1)          nu(2)          nu(3)            nu(4)
%      ------^------  ------^------  ------^------  --------^--------
%   D=[e_1, ..., e_1, e_2, ..., e_2, e_3, ..., e_3, e_123, ..., e_123]
%
%  to visualize either the s-set (s in Imm(M_D)) extraction of the 
%  Box-spline volume or the contour lines of the regions obtained by 
%  intersecting the Box-spline volume with three families of planes 
%  respectively parallel to yz, xz, xy.
%
%<Input Parameters>        
%   nu   --> 4-component vector with the multiplicities of the directions
%            e_1=[1 0 0]', e_2=[0 1 0]', e_3=[0 0 1]', e_123=[1 1 1]':
%            nu(1) multiplicity of direction e_1
%            nu(2) multiplicity of direction e_2
%            nu(3) multiplicity of direction e_3
%            nu(4) multiplicity of direction e_123
%
%   flag --> parameter that indicates the kind of visualization:
%            flag=1  isosurface of the Box-spline volume
%            flag=2  contour lines in slices of the Box-spline volume
%            flag=3  isosurface of the Box-spline volume and 
%                    contour lines in slices of the Box-spline volume
%            Default flag = 3.
%
%   m    --> parameter identifing the number of evaluation points (m+1) 
%            per tetrahedron edge (i.e. per unit cube edge).
%            Default m = 10.
%
%   s    --> parameter identifing the isosurface value at which computing
%            isosurface geometry for data ww, corresponding to the 
%            evaluations of the trivariate Box-spline over the
%            smallest parallelepiped containing its support region. 
%            Thus, s can be any value in the interval [0, max(ww)].  
%            Default s = 0.
%
%<See Also>
%  visual_mdds2d --> Visualization of any bivariate three-direction
%  Box-spline.

%<Algorithm>
%  First we compute the B-net coefficients of the trivariate four-direction
%  Box-spline associated with the multiplicity vector nu. Then we evaluate  
%  the trivariate Bernstein-Bezier polynomials on a regular tetrahedral grid
%  and compute the values of the Box-spline volume over each one of the six
%  tetrahedra obtained by considering the tessellation of the unit cube
%  proposed in Fig.3 (see [1]). Successively we exploit the Matlab functions
%  isosurface() and contourslice() to visualize respectively isosurfaces of
%  the Box-spline volume and contour lines of the regions obtained by
%  intersecting the Box-spline volume with three families of planes 
%  respectively parallel to yz, xz, xy.   
%
%<Reference>
%  [1] G. Casciola, E. Franchini, L. Romani, "The mixed directional 
%      difference-summation algorithm for generating the Bezier net of a 
%      trivariate four-direction Box-spline"
%      Numerical Algorithms 43 (2006), pp. 75-98.
%
%<Authors>
%  G. Casciola, Dept. of Mathematics - University of Bologna, Italy
%  E. Franchini, Dept. of Pure and Applied Mathematics - University of Padova, Italy
%  L. Romani, Dept. of Mathematics - University of Bologna, Italy
%
%  Last revision: March 22, 2006

% Check the required input arguments
if nargin < 1
    error('Not enough input arguments.')
elseif nargin > 4
    error('Too many input arguments.')    
end

% Check the optional input arguments, and set defaults
if nargin < 4
    s=0;
end
if nargin < 3
    m=10;
end
if nargin < 2
    flag=3;
end

switch flag
       case 1
            fig1=1;
            fig2=1;
       case 2
            fig1=2;
            fig2=2;
       case 3
            fig1=1;
            fig2=2;
end

% Box-spline degree
deg=sum(nu)-3;

% B-net computation
A=bnet_mdds(nu)./factorial(deg);

% Evaluation of trivariate Bernstein-Bezier polynomials on a regular 
% tetrahedral grid
bez=computebez_mdds3d(deg,m);

%Number of evaluation points in each tetrahedron
N=(m+1)*(m+2)*(m+3)/6;

for hz=0:nu(3)+nu(4)-1
    for hx=0:nu(1)+nu(4)-1
        for hy=0:nu(2)+nu(4)-1
            % Bezier coefficients of the six tetrahedral volumes contained  
            % in the cube region identified by indexes hx, hy, hz
            A1 = A(deg*hz+1:deg*hz+deg+1,deg*hx+1:deg*hx+deg+1, ...
                deg*hy+1:deg*hy+deg+1);
            %--------------------------------------------------------------
            % Tetrahedron "1"
            kk=0;
            for hh=deg+1:-1:1
                for jj=hh:deg+1
                    for ii=deg+2-hh:-1:deg+2-jj
                        kk=kk+1;
                        vw(kk)=A1(ii,jj,hh);
                    end
                end
            end
            for npt=1:N
                W(npt)=bez(npt,:)*vw';
            end
            kk=0;
            for hh=m+1:-1:1
                for jj=hh:m+1
                    for ii=m+2-hh:-1:m+2-jj
                        kk=kk+1;
                        W1(ii,jj,hh)=W(kk);
                    end
                end
            end
            %--------------------------------------------------------------
            % Tetrahedron "2"
            kk=0;
            for ii=1:deg+1
                for hh=1:ii
                    for jj=1:hh
                        kk=kk+1;
                        vw(kk)=A1(ii,deg+2-jj,deg+2-hh);
                    end
                end
            end
            for npt=1:N
                W(npt)=bez(npt,:)*vw';
            end
            kk=0;
            for ii=1:m+1
                for hh=1:ii
                    for jj=1:hh
                        kk=kk+1;
                        W1(ii,m+2-jj,m+2-hh)=W(kk);
                    end
                end
            end
            %--------------------------------------------------------------
            % Tetrahedron "3"
            kk=0;
            for ii=1:deg+1
                for hh=1:ii
                    for jj=1:hh
                        kk=kk+1;
                        vw(kk)=A1(ii,deg+2-hh,deg+2-jj);
                    end
                end
            end
            for npt=1:N
                W(npt)=bez(npt,:)*vw';
            end
            kk=0;
            for ii=1:m+1
                for hh=1:ii
                    for jj=1:hh
                        kk=kk+1;
                        W1(ii,m+2-hh,m+2-jj)=W(kk);
                    end
                end
            end
            %--------------------------------------------------------------
            % Tetrahedron "4"
            kk=0;
            for jj=deg+1:-1:1
                for hh=jj:deg+1
                    for ii=deg+2-jj:-1:deg+2-hh
                        kk=kk+1;
                        vw(kk)=A1(ii,jj,hh);
                    end
                end
            end
            for npt=1:N
                W(npt)=bez(npt,:)*vw';
            end
            kk=0;
            for jj=m+1:-1:1
                for hh=jj:m+1
                    for ii=m+2-jj:-1:m+2-hh
                        kk=kk+1;
                        W1(ii,jj,hh)=W(kk);
                    end
                end
            end
            %--------------------------------------------------------------
            % Tetrahedron "5"       
            kk=0;
            for ii=deg+1:-1:1
                for jj=deg+2-ii:-1:1
                    for hh=deg+3-ii-jj:-1:1
                        kk=kk+1;
                        vw(kk)=A1(ii,jj,hh+jj-1);
                    end
                end
            end
            for npt=1:N
                W(npt)=bez(npt,:)*vw';
            end
            kk=0;
            for ii=m+1:-1:1
                for jj=m+2-ii:-1:1
                    for hh=m+3-ii-jj:-1:1
                        kk=kk+1;
                        W1(ii,jj,hh+jj-1)=W(kk);
                    end
                end
            end
            %--------------------------------------------------------------
            % Tetrahedron "6"
            kk=0;
            for ii=deg+1:-1:1
                for hh=deg+2-ii:-1:1
                    for jj=deg+3-ii-hh:-1:1
                        kk=kk+1;
                        vw(kk)=A1(ii,jj+hh-1,hh);
                    end
                end
            end
            for npt=1:N
                W(npt)=bez(npt,:)*vw';
            end
            kk=0;
            for ii=m+1:-1:1
                for hh=m+2-ii:-1:1
                    for jj=m+3-ii-hh:-1:1
                        kk=kk+1;
                        W1(ii,jj+hh-1,hh)=W(kk);
                    end
                end
            end
            %--------------------------------------------------------------
            ww(m*hz+1:m*(hz+1)+1,m*hx+1:m*(hx+1)+1,m*hy+1:m*(hy+1)+1)=W1;
            % ww is a tri-dimensional array with the evaluations of the  
            % trivariate Box-spline over the smallest parallelepiped 
            % containing its support region. 
        end
    end
end      

M=max(max(max(ww)));
if ((flag==1 | flag==3) & nargin==4)
    while (s<0 | s>=M)
          fprintf('The parameter s must be contained in the interval\n');
          fprintf('0 <= s < %f\n', M);
          s=input('Insert s --> ');
    end
end

x=linspace(0,nu(1)+nu(4),m*(nu(1)+nu(4))+1);
y=linspace(0,nu(2)+nu(4),m*(nu(2)+nu(4))+1);
z=linspace(nu(3)+nu(4),0,m*(nu(3)+nu(4))+1);

% size(ww)=[deg(nu(3)+nu(4))+1]x[deg(nu(1)+nu(4))+1]x[deg(nu(2)+nu(4))+1]
[xw,zw,yw]=meshgrid(x,z,y);

[rnu,cnu]=size(nu);
if cnu==1
   nu=nu';
end

for i=fig1:fig2
    if i==1
       
       % Visualization of the isosurface corresponding to the value s 
       % (s in [0,max(ww)])
       figure(i)
       clf
       [F V] = isosurface(xw,yw,zw,ww,s);
       q=trisurf(F,V(:,1),V(:,2),V(:,3));
       shading faceted
       colormap(copper)
       isonormals(xw,zw,yw,ww,q);
       title(['Box-spline isosurface  nu=[',num2str(nu),']']);
       axis([-1 nu(1)+nu(4)+1 -1 nu(2)+nu(4)+1 -1 nu(3)+nu(4)+1]); 
       xlabel('x'); ylabel('y'); zlabel('z');
       axis vis3d        
    else    
       
       % Visualization of the contour lines of the regions obtained by   
       % intersecting the Box-spline volume with three families of planes  
       % respectively parallel to the coordinate planes yz, xz, xy. 
       figure(i)
       clf
       subplot(311);
       title(['Box-spline nu=[',num2str(nu),']',' - Countourslice x']);
       hold on;
       contourslice(xw,zw,yw,ww,0:0.2:x(end),[],[]);
       colormap(cool)
       subplot(312);
       title(['Box-spline nu=[',num2str(nu),']',' - Countourslice y']);
       hold on;
       contourslice(xw,zw,yw,ww,[],[],0:0.2:y(end));
       colormap(cool)
       subplot(313);
       title(['Box-spline nu=[',num2str(nu),']',' - Countourslice z']);
       hold on;
       contourslice(xw,zw,yw,ww,[],z(1):-0.2:z(end),[]);
       colormap(cool)
    end
end
